/*~******************************************************************************************
*	~Project:	"Alone"																		*
*	~Author:	Tim Beaudet																	*
*	~Modified:	12/16/2011																	*
*	~Purpose:	Common includes, defines and constants, and main							*
******************************************************************************************~*/

#include "Alone.h"

#include "CStateMachine.h"
#include "GameStates/CSplashScreen.h"
#include "GameStates/CMainMenu.h"
#include "GameStates/CTutorial.h"
#include "GameStates/CGamePlay.h"

#ifdef _DEBUG
  #pragma comment(lib, "DebugToolSetDbg.lib")
  #pragma comment(lib, "InternalCombustionDbg.lib")
#else
  #pragma comment(lib, "DebugToolSet.lib")
  #pragma comment(lib, "InternalCombustion.lib")
#endif

#define WIN32_LEAN_AND_MEAN
#include <windows.h>

std::string gToDataDir = "data/";
std::string gToUserDir = "user/";

void SetDefaultConfig(Config *cfg)
{
	cfg->mWindowWidth = 1024;	cfg->mWindowHeight = 768;
	cfg->mWindowPosX = 100;		cfg->mWindowPosY = 100;
	cfg->mFullScreen = false;

	cfg->mGameTime = 120;
	cfg->mAwayPoints = 12;
}

void LoadConfigFromFile(Config *cfg, const std::string &strFilePath)
{
	SetDefaultConfig(cfg);
	Utilities::CScript *file = Utilities::OpenScriptFromFile(strFilePath);
	if (nullptr != file)
	{
		std::string token;
		while (false == file->EndOfFile())
		{
			token = file->Read();
			for (std::string::iterator itr = token.begin(), itrEnd = token.end(); itr != itrEnd; ++itr)
				*itr = tolower(*itr);

			if ("window" == token)
			{
				cfg->mWindowWidth = file->ReadAs<int>();
				cfg->mWindowHeight = file->ReadAs<int>();
				cfg->mWindowPosX = file->ReadAs<int>();
				cfg->mWindowPosY = file->ReadAs<int>();
				cfg->mFullScreen = file->ReadAs<bool>();
			}
			if ("awaypoints" == token)
				cfg->mAwayPoints = file->ReadAs<int>();
			if ("gametime" == token)
				cfg->mGameTime = file->ReadAs<int>();
		}

		Utilities::CloseScript(file);
	}
}

void SaveConfigToFile(Config *cfg, const std::string &strFilePath)
{
	Utilities::CScript *file = Utilities::OpenScriptFromData("");
	if (nullptr != file)
	{
		file->Write("Window ");
		file->WriteAs(cfg->mWindowWidth);	file->WriteAs(' ');
		file->WriteAs(cfg->mWindowHeight);	file->WriteAs(' ');
		file->WriteAs(cfg->mWindowPosX);		file->WriteAs(' ');
		file->WriteAs(cfg->mWindowPosY);		file->WriteAs(' ');
		file->WriteAs(cfg->mFullScreen);		file->WriteAs(' ');
		file->Write("GameTime ");	file->WriteAs(cfg->mGameTime);	file->WriteAs('\n');
		file->Write("AwayPoints ");	file->WriteAs(cfg->mAwayPoints);		file->WriteAs('\n');

		file->SaveToFile(strFilePath);
		Utilities::CloseScript(file);
	}
}

const std::string& ToDataDir(void) {	return gToDataDir;	}
const std::string& ToUserDir(void) {	return gToUserDir;	}

bool frameFunc(void)
{
	if (false == TheStateMachine->IsStateValid())
		return false;
	
	float deltaTime = (float)TheTimingSystem->DeltaTime();

	//-------------------------------------	Simulate
	static float simulateTime = 0.0f;
	simulateTime += deltaTime;
	while (simulateTime > SimulateTimeSec)
	{
		TheStateMachine->Simulate();
		simulateTime -= SimulateTimeSec;
	}

	//-------------------------------------	Update
	TheStateMachine->Update(deltaTime);

	if (false == TheStateMachine->DoesStateSetupScene())
	{
		//---------------------------------	Render
		TheVisualSystem->Clear(0xFF000000);
		TheVisualSystem->BeginScene();
		TheStateMachine->Render();
		TheVisualSystem->EndScene();

		//---------------------------------	Draw
		TheVisualSystem->BeginDraw();
		TheStateMachine->Draw();
		TheVisualSystem->EndDraw();	

		TheVisualSystem->Present();
	}
	else
	{
		DTS_NOTIMPLEMENTED;
	}

	return true;
}


namespace Textures
{
	void LoadTextures(void);
	void UnloadTextures(void);
};

namespace Sounds
{
	void LoadSounds(void);
	void UnloadSounds(void);

	void Update(void);
};

void realMain(void)
{
	LoadConfigFromFile(TheConfig, ToUserDir() + "config.cfg");

	//System Setup.
	SetSystemFunc(ICE_STATE_FRAMEFUNC, frameFunc);
	SetSystemString(ICE_STATE_WINDOWTITLE, "Ten Yards Alone");
	SetSystemInt(ICE_STATE_WINDOWPOSX, TheConfig->mWindowPosX);
	SetSystemInt(ICE_STATE_WINDOWPOSY, TheConfig->mWindowPosY);
	SetSystemInt(ICE_STATE_WINDOWWIDTH, TheConfig->mWindowWidth);
	SetSystemInt(ICE_STATE_WINDOWHEIGHT, TheConfig->mWindowHeight);
	SetSystemBool(ICE_STATE_FULLSCREEN, TheConfig->mFullScreen);
	SetSystemBool(ICE_STATE_VSYNC, false);

	if (true == CreateInternalCombustion())
	{	//Initalize Project
		Textures::LoadTextures();
		Sounds::LoadSounds();

		IState *startState = nullptr;
		IState *splashState = nullptr;
		IState *menuState = nullptr;
		TheStateMachine->AddState(startState = splashState = new CSplashScreen());
		TheStateMachine->AddState(menuState = new CMainMenu());
		TheStateMachine->AddState(new CTutorial());
		TheStateMachine->AddState(startState = new CGamePlay());
		

#ifndef _DEBUG
		startState = splashState;
#endif

		TheStateMachine->ChangeToState(startState->StateName());

		//Run Project
		TheSystem->RunInternalCombustion();

		//Cleanup Project
		SaveConfigToFile(TheConfig, ToUserDir() + "config.cfg");
		Textures::UnloadTextures();
		Sounds::UnloadSounds();
		DestroyInternalCombustion();
	}

	DTS_LOG_TEXT("Game is over.");
}

int CALLBACK WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, char *cmdLine, int nShow)
{
	DTS_LOG_INIT("Alone.log", true);
	DTS_PROJECT_ENTRY_POINT(realMain);
	DTS_LOG_QUIT;
	return 0;
}



namespace Textures
{
	iceTexture* FieldTiles = nullptr;
	iceTexture* AwayPlayer = nullptr;
	iceTexture* HomePlayer = nullptr;
	iceTexture* HeadsUpDisplay = nullptr;
	iceTexture* SimpleFont = nullptr;
	iceTexture* DustBall = nullptr;
	iceTexture* Football = nullptr;
	iceTexture* HomeTeam = nullptr;


	void LoadTextures(void)
	{
		TheVisualSystem->Texture_CreateFromFile(&FieldTiles, ToDataDir() + "tileset.png");
		TheVisualSystem->Texture_CreateFromFile(&HomePlayer, ToDataDir() + "HomePlayer.png");
		TheVisualSystem->Texture_CreateFromFile(&AwayPlayer, ToDataDir() + "AwayPlayer.png");
		TheVisualSystem->Texture_CreateFromFile(&HeadsUpDisplay, ToDataDir() + "hud.png");
		TheVisualSystem->Texture_CreateFromFile(&SimpleFont, ToDataDir() + "simplefont.png");		
		TheVisualSystem->Texture_CreateFromFile(&DustBall, ToDataDir() + "dustball.png");		
		TheVisualSystem->Texture_CreateFromFile(&Football, ToDataDir() + "football.png");		
		TheVisualSystem->Texture_CreateFromFile(&HomeTeam, ToDataDir() + "HomeTeam.png");		
	}

	void UnloadTextures(void)
	{
		TheVisualSystem->Texture_Destroy(&FieldTiles);		FieldTiles = nullptr;
		TheVisualSystem->Texture_Destroy(&HomePlayer);		HomePlayer = nullptr;
		TheVisualSystem->Texture_Destroy(&AwayPlayer);		AwayPlayer = nullptr;
		TheVisualSystem->Texture_Destroy(&HeadsUpDisplay);	HeadsUpDisplay = nullptr;
		TheVisualSystem->Texture_Destroy(&SimpleFont);		SimpleFont = nullptr;
		TheVisualSystem->Texture_Destroy(&DustBall);			DustBall = nullptr;
		TheVisualSystem->Texture_Destroy(&Football);			Football = nullptr;
		TheVisualSystem->Texture_Destroy(&HomeTeam);			HomeTeam = nullptr;
	}

};

namespace Sounds
{
	iceAudio *Music = nullptr;
	iceAudio *VoiceOver = nullptr;
	iceAudio *SoundEffect = nullptr;
	iceAudio *Whistle = nullptr;
	iceAudio *HikeBall = nullptr;
	//
	//struct voq { std::string vfx; int maxRand; };
	//std::list<voq> VFX_Q;
	//void QueueVO(const std::string &vfx, int maxRand)
	//{

	//}

	//void Update(void)
	//{
	//	if (true == VFX_Q.empty())
	//		return;

	//	if (nullptr == VoiceOver || true == TheAudioSystem->T
	//}

	void PlayVO(const std::string &vfx, int maxRand)
	{
		if (nullptr != VoiceOver)
			TheAudioSystem->TestSound_Destroy(&VoiceOver);

		std::string file = ToDataDir() + "/vfx/" + vfx;
		if (maxRand > 0)
		{
			char buff[10];
			_itoa(Rand::Rand() % maxRand, buff, 10);
			file += buff;
			file += ".wav";
		}
		else if (maxRand == 0)
			file += "0.wav";
		else
			file += ".wav";
		
		TheAudioSystem->TestSound_CreateFromFile(&VoiceOver, file, ICEAF_BUFFERED);
		TheAudioSystem->TestSound_Play(VoiceOver, false);
	}

	void PlaySfx(const std::string &vfx, int maxRand)
	{
		if (nullptr != SoundEffect)
			TheAudioSystem->TestSound_Destroy(&SoundEffect);

		std::string file = ToDataDir() + "/sfx/" + vfx;
		if (maxRand > 0)
		{
			char buff[10];
			_itoa(Rand::Rand() % maxRand, buff, 10);
			file += buff;
			file += ".wav";
		}
		else if (maxRand == 0)
			file += "0.wav";
		else
			file += ".wav";
		
		TheAudioSystem->TestSound_CreateFromFile(&SoundEffect, file, ICEAF_BUFFERED);
		TheAudioSystem->TestSound_Play(SoundEffect, false);
	}

	void LoadSounds(void)
	{
		TheAudioSystem->TestSound_CreateFromFile(&Music, ToDataDir() + "music.wav", ICEAF_STREAMED);
		TheAudioSystem->TestSound_CreateFromFile(&Whistle, ToDataDir() + "/vfx/whistle0.wav", ICEAF_BUFFERED);
		TheAudioSystem->TestSound_CreateFromFile(&HikeBall, ToDataDir() + "/vfx/hike0.wav", ICEAF_BUFFERED);
	}

	void UnloadSounds(void)
	{
		TheAudioSystem->TestSound_Destroy(&Music);		Music = nullptr;
		TheAudioSystem->TestSound_Destroy(&Whistle);		Whistle = nullptr;
		TheAudioSystem->TestSound_Destroy(&HikeBall);	HikeBall = nullptr;

		if (nullptr != VoiceOver)
		{
			TheAudioSystem->TestSound_Destroy(&VoiceOver);
			VoiceOver = nullptr;
		}

		if (nullptr != SoundEffect)
		{
			TheAudioSystem->TestSound_Destroy(&SoundEffect);
			SoundEffect = nullptr;
		}
	}
};

namespace Rand
{
	void SetSeed(unsigned long seed) { srand(seed); }
	unsigned long Rand(void) { return (unsigned long)rand(); }
	float RandF(void) { return (rand() % 10000) / 10000.0f; }
	float RandF(float min, float max) { return ((max - min) * RandF()) + min; }
};