/*~******************************************************************************************
*	~Project:	"Alone"																		*
*	~Author:	Tim Beaudet																	*
*	~Modified:	12/16/2011																	*
*	~Purpose:	Football field for some basic gameplay.										*
******************************************************************************************~*/

#include "Alone.h"
#include "CFootballField.h"
#include "CFootballPlayer.h"
#include "CFootball.h"

#include "CStateMachine.h"

const int FIRSTDOWN_LINE = 9;
const int SCRIMMAGE_LINE = 8;

class CDustBall
{
private:
	float mLifeTime;
	CSpriteObject mSprite;

public:
	CDustBall(void) : mLifeTime(-1.0f)
	{
		mSprite.mWidth = mSprite.mHeight = 2.0f;
		mSprite.SetTexture(Textures::DustBall);
	}

	void Spawn(const Vector3 &at)
	{
		mLifeTime = 2.0f;
		mSprite.SetPosition(at);
	}

	void Update(float dt)
	{
		if (mLifeTime >= 0.0f)
		{
			mLifeTime -= dt;
			unsigned char a = (unsigned char)(( (mLifeTime / 2.0f)) * 256.0f);
			mSprite.GetSprite().SetColor(0x00FFFFFF | (a << 24));
		}
	}

	void Draw(void) const
	{
		if (mLifeTime > 0.0f)
			mSprite.Draw();
	}
};



static const float PopupFade = 0.75f;

struct InfoLine
{
	std::string text;
	float x, y, size;
};

class CPopupInfo : public CSpriteObject
{
private:
	bool mIsOpen;
	float mFadeTimer;
	float mMinOpenTimer;
	bool mShowPressSpace;
	CSpriteFont mFont;

	std::vector<InfoLine> *mCurrLines;
	std::list<std::vector<InfoLine>> mQueuedPopups;

public:
	float LeftEdge(void) const { return GetPosition().x - (mWidth * 0.5f) + 0.01f; }
	float RightEdge(void) const { return GetPosition().x + (mWidth * 0.5f) - 0.01f; }
	float CenterX(void) const { return GetPosition().x; }

	float BottomEdge(void) const { return GetPosition().y + (mHeight * 0.5f) - 0.01f; }
	float TopEdge(void) const { return GetPosition().y - (mHeight * 0.5f) + 0.01f; }
	float CenterY(void) const { return GetPosition().y; }

	float CenterOfX(const std::string &str, float size) { return CenterX() - (mFont.ComputeWidth(size, str.c_str()) * 0.5f); }

public:

	CPopupInfo(void) : mIsOpen(false), mFadeTimer(-1.0f), mMinOpenTimer(0.0f), mShowPressSpace(true),
		mCurrLines(nullptr)
	{
		SetTexture(Textures::FieldTiles);
		GetSprite().SetTextureCoords(0.8751f, 0.2501f, 0.1249f, 0.1249f);

		float aspect = TheVisualSystem->GetAspectRatio();
		SetPosition(Vector3(0.5f * aspect, 0.5f, 0.0f));
		GetSprite().SetColor(0xFF202030);
		mWidth = 0.75f;
		mHeight = mWidth * 0.5f;
		mWorldSpace = false;
		AddPage(false);
		mCurrLines = &mQueuedPopups.front();
	}

	void Update(float dt)
	{
		if (true == mIsOpen && mFadeTimer < PopupFade)
		{
			mFadeTimer += dt;
			if (mFadeTimer > PopupFade)
				mFadeTimer = PopupFade;
		}
		else if (false == mIsOpen && mFadeTimer > 0.0f)
		{
			mFadeTimer -= dt;
			if (mFadeTimer < 0.0f)
				mFadeTimer = -1.0f;
		}

		if (mMinOpenTimer < 5.0f)
			mMinOpenTimer += dt;

		float p = (mFadeTimer / PopupFade);
		unsigned char a = (unsigned char)(p * 255.0f);
		GetSprite().SetColor(0x00202030 | (a << 24) );//& 0x80FFFFFF);
		mFont.SetColor(0x00FFFFFF | (a << 24));
	}

	virtual void Draw(void) const
	{
		if (false == IsOpen() || nullptr == mCurrLines)
			return;

		CSpriteObject::Draw();

		TheVisualSystem->Texture_Bind(0, Textures::SimpleFont);
		if (true == mShowPressSpace)
			mFont.Draw(CenterX() - (CSpriteFont::ComputeWidth(0.04f, "Press space to continue") * 0.5f), BottomEdge() - 0.05f, 	0.04f, "Press space to continue");

		for (std::vector<InfoLine>::const_iterator itr = mCurrLines->begin(), itrEnd = mCurrLines->end(); itr != itrEnd; ++itr)
			mFont.Draw(itr->x, itr->y, itr->size, itr->text.c_str());
	}

	InfoLine& AddLine(const std::string &text, float size, bool leftAligned = true)
	{
		InfoLine retLine;	retLine.text = text;	retLine.size = size;		

		if (true == leftAligned)
			retLine.x = LeftEdge();
		else
			retLine.x = CenterOfX(retLine.text, retLine.size);

		if (true == mQueuedPopups.back().empty())
		{
			retLine.y = TopEdge();
		}
		else
		{
			InfoLine &lastLine = mQueuedPopups.back().back();
			retLine.y = lastLine.y + lastLine.size + 0.01f;
		}

		return AddLine(retLine);
	}

	InfoLine& AddLine(const InfoLine &addLine)
	{
		DTS_LOG_TEXT("Add line to page %d\n", mQueuedPopups.size());
		mQueuedPopups.back().push_back(addLine);
		return mQueuedPopups.back().back();
	}

	void ClearPages(void)
	{
		mQueuedPopups.clear();
		AddPage(false);
		mCurrLines = &mQueuedPopups.front();
	}

	void AddPage(bool openNow = true)
	{
		DTS_LOG_TEXT("Add page\n");
		std::vector<InfoLine> v;
		mQueuedPopups.push_back(v);
		if (true == openNow)
			Open();
	}

	//0 = current page [tossed at flip.
	//1 = page being worked on.  It would take page 1 to be finished with more behind for ready.
	bool HasPageReady(void) { return (mQueuedPopups.size() > 2) ? true : false; }
	void FlipPage(void) 
	{ 
		DTS_LOG_TEXT("Flip page\n");
		mQueuedPopups.pop_front();
		mCurrLines = &mQueuedPopups.front();
		Open();
	}

	void Open(void) 
	{ 
		DTS_LOG_TEXT("Open\n");
		mIsOpen = true; 
		if (mFadeTimer < 0.0f) 
			mFadeTimer = 0.0f; 
		mMinOpenTimer = 0.0f; 
	}

	void Close(void)	{ mIsOpen = false; }
	bool IsOpen(void) const { return (true == mIsOpen || mFadeTimer > 0.0f) ? true : false; }
	bool CanClose(void) const { return (mMinOpenTimer > 1.0f) ? true : false; }
};


CFootballField::CFootballField(void) : mGameTimeLeft(0.0f), mAwayScore(0), mHomeScore(0),
	mIsInPlay(false), mReadyToPlay(false), mIsScoring(false), mDustBall(new CDustBall()),
	mFootball(new CFootball()), mPopupInfo(new CPopupInfo()), mHasGameConcluded(false)
{
	mCols = 60;		mRows = 150;
	mRowOfYard1 = 24;
	mColOfLeftEdge = 20;
	mColOfRightEdge = 39;

	mTiles = new char[mCols * mRows];
	FILE *file = fopen((ToDataDir() + "field.raw").c_str(), "rb");

	if (file)
	{
		fread(mTiles, mCols * mRows, 1, file);
		fclose(file);
	}

	mFootball->SetFootballField(this);
}

CFootballField::~CFootballField(void)
{
	delete [] mTiles;		mTiles = nullptr;
	delete mDustBall;		mDustBall = nullptr;
	delete mFootball;		mFootball = nullptr;
	delete mPopupInfo;		mPopupInfo = nullptr;
}

void CFootballField::Update(float dt)
{
	mPopupInfo->Update(dt);
	mDustBall->Update(dt);
	mFootball->Update(dt);

	if ((true == mIsScoring || true == mPopupInfo->IsOpen()) && true == mPopupInfo->CanClose())
	{	//Should be showing a report of loss/gained yards, FD etc...
		if (true == TheInputSystem->IsKeyPressed(ICEKEY_SPACE))
		{			
			mPopupInfo->Close();
			if (true == mPopupInfo->HasPageReady())
			{
				mPopupInfo->FlipPage();
				return;
			}
			else
			{
				mPopupInfo->ClearPages();
				if (mGameTimeLeft < 0.0f && true == mHasGameConcluded)
					TheStateMachine->ChangeToState("Credits");

				ResetForDown();
				return;
			}
		}
		
		return;//?
	}

	if (mGameTimeLeft >= 0.0f)
	{
		mGameTimeLeft -= dt;
		if (mGameTimeLeft < 0.0f)
		{
			Sounds::PlaySfx("timeup");
			mGameTimeLeft = -1.0f;
			if (false == mIsInPlay && true == mReadyToPlay)
			{
				OnGameEnd();
				return;
			}
		}
	}

	if (false == mIsInPlay && true == mReadyToPlay)
	{
		Vector3 dist;
		Vector3Subtract(&dist, &mPlayerStart, &GetPlayerPosition());
		if (Vector3MagnitudeSq(&dist) > 0.001f)
			HikeFootball();
	}

	if (true == mIsInPlay)
	{
		bool wasTackled = false;
		bool playerHadBall = false;
		Vector3 dist;
		const CFootballPlayer *plyrWithBall = GetPlayerWithFootball();
		if (nullptr != plyrWithBall)
		{
			const Vector3 &playerPos = plyrWithBall->GetPosition();
			for (PlayerCItr itr = mPlayers.begin(), itrEnd = mPlayers.end(); itr != itrEnd; ++itr)
			{
				CFootballPlayer *opp = *itr;
				if (TEAM_HOME == opp->GetTeam())
					continue;

				float tackleDist = (plyrWithBall->mRadius + opp->mRadius) * 0.5f;
				Vector3Subtract(&dist, &playerPos, &opp->GetPosition());
				if (Vector3MagnitudeSq(&dist) > (tackleDist * tackleDist))
					continue;

				wasTackled = true;
				break;
			}
		}

		if (mPlayers[0] == plyrWithBall)
			playerHadBall = true;

		const Vector3 &playPos = GetPlayerPosition();
		if (true == playerHadBall && playPos.z < mRowOfYard1 - 0.0f)
		{
			Sounds::PlayVO("anc_touchdown", 2);
			EndOfDown(DOWN_TOUCHDOWN);
		}
		else if (playPos.x < (mColOfLeftEdge * 1.0f + 0.5f) || playPos.x > (mColOfRightEdge * 1.0f - 0.5f))
		{
			Sounds::PlayVO("anc_outbound", 2);
			EndOfDown(DOWN_OUTBOUNDS);
		}
		else if (true == playerHadBall && playPos.z > mRowOfYard1 + 110.0f)
		{
			Sounds::PlayVO("anc_safety", 0);
			EndOfDown(DOWN_SAFETY);
		}
		else if (nullptr == plyrWithBall && false == mFootball->IsInAir())
		{
			Sounds::PlayVO("anc_incomplete", 3);
			EndOfDown(DOWN_INCOMPLETE);
		}
		else if (nullptr != plyrWithBall && TEAM_AWAY == plyrWithBall->GetTeam())
		{
			Sounds::PlayVO("anc_intercept", 3);
 			EndOfDown(DOWN_INTERCEPT);
		}
		else if (true == wasTackled || true == TheInputSystem->IsKeyPressed(ICEKEY_ENTER))
		{
			if (true == playerHadBall)
			{	//Player was tackled.
				Sounds::PlayVO("anc_contact", 5);
			}
			else
			{	//Not the player though, the QB got sacked				
				Sounds::PlayVO("anc_qbsacked", 0);
			}

 			EndOfDown(DOWN_CONTACT);
		}
	}

	for (PlayerItr itr = mPlayers.begin(), itrEnd = mPlayers.end(); itr != itrEnd; ++itr)
		(*itr)->Update(dt);
}

void CFootballField::Draw(void) const
{
	float aspect = TheVisualSystem->GetAspectRatio();
	Vector3 camPos = mPlayers[0]->GetPosition();
	camPos.z -= 3.0f;	//View upfield a bit
	Vector3 offset(0.5f * aspect, 0.5f, 0.0f);		Camera::ToWorldScale(&offset);	
	Vector3Subtract(&camPos, &camPos, &offset);
	Camera::SetPosition(camPos);

	//Hopefully the smart way, draw what is seen.
	int leftEdge= (int)camPos.x - 2;
	int topEdge = (int)camPos.z - 2;
	for (int x = leftEdge; x <= leftEdge + 30; ++x)
	{
		if (x < 0 || x > mCols)
			continue;
	
		for (int y = topEdge; y <= topEdge + 20; ++y)
		{
			if (y < 0 || y > mRows)
				continue;

			DrawTile(mTiles[x + (y * mCols)], Vector3(1.0f * x, 0.0f, 1.0f * y));
		}
	}

	if (mLineOfScrimmage > 0 && mLineOfScrimmage < 100) DrawMarkerLine(mLineOfScrimmage, SCRIMMAGE_LINE);
	if (mNextFirstDown > 0)	DrawMarkerLine(mNextFirstDown, FIRSTDOWN_LINE);

	for (PlayerCItr itr = mPlayers.begin(), itrEnd = mPlayers.end(); itr != itrEnd; ++itr)
		(*itr)->Draw();

	mFootball->Draw();
	mDustBall->Draw();

	/////////  Score and timer stuff
	mPopupInfo->Draw();

	CSprite hudBG;
	//hudBG.SetTexture(Textures::HeadsUpDisplay);
	TheVisualSystem->Texture_Bind(0, Textures::HeadsUpDisplay);
	float hudSize = 1.0f;
	float hudLeft = (0.5f * aspect) - (hudSize * 0.5f);
	float hudTop = 1.0f - (hudSize * 0.125f);
	hudBG.SetScreenCoords(hudLeft, hudTop, hudSize, hudSize * 0.125f);
	hudBG.Draw();

	TheVisualSystem->Texture_Bind(0, Textures::SimpleFont);
	CSpriteFont fnt;
	fnt.DrawFmt(hudLeft + (hudSize * 0.18f), 0.902f, 0.0325f, "%d", mHomeScore);
	fnt.DrawFmt(hudLeft + (hudSize * 0.18f), 0.948f, 0.0325f, "%d", mAwayScore);

	char down = (mCurrentDown < 1) ? 1 : mCurrentDown;
	float len = fnt.ComputeWidthFmt(0.07f, "%d", down);
	fnt.DrawFmt(hudLeft + (hudSize * 0.5f) - (len * 0.5f), hudTop + 0.0625f - 0.035f, 0.07f, "%d", down);
	char yardsToGo = (0 == mNextFirstDown) ? mLineOfScrimmage : mLineOfScrimmage - mNextFirstDown;
	len = fnt.ComputeWidthFmt(0.0325f, "and %d", yardsToGo);
	fnt.DrawFmt(hudLeft + (hudSize * 0.52f) - (len * 0.5f), 0.965f, 0.0325f, "and %d", yardsToGo);
	
	float gameTime = (mGameTimeLeft < 0.0f) ? 0.0f : mGameTimeLeft;
	len = fnt.ComputeWidthFmt(0.0325f, "%.0f", gameTime);
	fnt.DrawFmt(hudLeft + (hudSize * 0.825f) - len, 0.902f, 0.0325f, "%.0f", gameTime);
	fnt.DrawFmt(hudLeft + (hudSize * 0.725f), 0.948f, 0.0325f, "To Goal: %d", 100);
}

void CFootballField::Reset4thQuarter(float timeLeft, unsigned char awayScore, char lineOfScrim)
{
	mGameTimeLeft = timeLeft;
	mAwayScore = awayScore;
	mHomeScore = 0;	
	mLineOfScrimmage = lineOfScrim;
	mBallPlacement = lineOfScrim;
	ComputeFirstDown();
	mCurrentDown = 0;
	ClearFieldAfterPlay();
	mIsScoring = false;
	mHasGameConcluded = false;

	mTotalYardage = 0;
	mTotalDownsPlayed = 0;
	mIncompletePasses = 0;
	mTotalInterceptions = 0;
	mTotalPasses = 0;
}

PlayerType RandAi(void)
{
	return PLYR_AWAY_INDIRECT;
	return PlayerType(PLYR_AWAY_DIRECT + Rand::Rand() % 2);
}

void CFootballField::ResetForDown()
{
	if (mGameTimeLeft < 0.0f)
	{
		OnGameEnd();
	}

	ClearFieldAfterPlay();

	mLineOfScrimmage = mBallPlacement;
	
	mCurrentDown++;	
	CFootballPlayer *ballCarrier = nullptr;
	Vector3 pos = GetCenterAt(mLineOfScrimmage);
	mPlayerStart = pos;
	CFootballPlayer *ply = PlacePlayer(PLYR_ALONE_MAN, TEAM_HOME, mPlayerStart);
	pos.x -= 1.2f;		PlacePlayer(PLYR_HOME_TBLOCK, TEAM_HOME, pos);
	pos.x += 2.4f;		PlacePlayer(PLYR_HOME_TBLOCK, TEAM_HOME, pos);
	
	pos = GetCenterAt(mLineOfScrimmage);
	pos.z += 2.1f;		CFootballPlayer *qb = PlacePlayer(PLYR_HOME_QB, TEAM_HOME, pos);
	pos.z -= 4.0f;
	pos.x += 1.2f;		PlacePlayer(PLYR_HOME_BLOCK, TEAM_HOME, pos);
	pos.x -= 4.8f;		PlacePlayer(PLYR_HOME_BLOCK, TEAM_HOME, pos);


	ballCarrier = ply;
	ballCarrier->SetFootball(mFootball);
	mFootball->SetPlayer(ballCarrier);

	//AWAY TEAM!
	pos = GetCenterAt(mLineOfScrimmage);
	pos.z -= 1.75f;		//PlacePlayer(PLYR_AWAY_DIRECT, TEAM_AWAY, pos);
	pos.x -= 1.98f;		PlacePlayer(RandAi(), TEAM_AWAY, pos);
	pos.x += 3.98f;		PlacePlayer(RandAi(), TEAM_AWAY, pos);

	pos = GetCenterAt(mLineOfScrimmage);	
	pos.z -= 6.0f;		PlacePlayer(RandAi(), TEAM_AWAY, pos);
	pos.x -= 3.0f;		PlacePlayer(RandAi(), TEAM_AWAY, pos);
	pos.x += 6.0f;		PlacePlayer(RandAi(), TEAM_AWAY, pos);
	pos.z += 1.5f;
	pos.x += 3.0f;		PlacePlayer(RandAi(), TEAM_AWAY, pos);
	pos.x -= 12.0f;		PlacePlayer(RandAi(), TEAM_AWAY, pos);


	mReadyToPlay = true;
	mIsScoring = false;
	DTS_LOG_TEXT("Field ready; Down: %d and %d on the %d\n", mCurrentDown, mLineOfScrimmage - mNextFirstDown, mLineOfScrimmage);
}

void CFootballField::ResetAfterPunt()
{
	mCurrentDown = 0;
	mBallPlacement = 80 + rand() % 18;	//TODO: Wonder about random ...
	ComputeFirstDown();
	//ResetForDown();		Will be called when 'scoring' is done; player hits space.
}

void CFootballField::EndOfDown(DownReasons reason)
{
	static std::string strDown[DOWN_MAX] = { 
		"Down by contact", "Interception!", "Out of Bounds", "Incomplete Pass",
		"Touch Down", "Safety", };
		
	char stoppedAt = FieldPosition(GetFootballPosition().z);
	char numYardsGained = mLineOfScrimmage - stoppedAt;

	if (reason == DOWN_INTERCEPT || reason == DOWN_INCOMPLETE)
	{
		numYardsGained = 0;

		if (reason == DOWN_INTERCEPT)
			mTotalInterceptions++;
		if (reason == DOWN_INCOMPLETE)
			mIncompletePasses++;
	}
	
	mBallPlacement = mLineOfScrimmage - numYardsGained;
	mTotalYardage += numYardsGained;
	mTotalDownsPlayed++;
	bool gainedYards = numYardsGained >= 0;

	//if (stoppedAt <= 0 && true == validBallPos)
	if (mBallPlacement <= 0)
	{	//TOUCH DOWN
		OnTouchDown();
		ResetAfterPunt();
	}
	else if (mBallPlacement <= mNextFirstDown)
	{	//The player managed to get a first down.
		mCurrentDown = 0;
		ComputeFirstDown();
	}
	else if (mBallPlacement > 99)
	{	//The player was sacked in the opposite endzone.
		OnSafety();
		ResetAfterPunt();
	}
	else if (mCurrentDown == 4)
	{	//The player went for it on 4th down, and did not make it.
		DTS_LOG_FAIL("Fourth Down Failed.");
		mGameTimeLeft -= 30.0f;
		if (mBallPlacement > 80)
		{
			mPopupInfo->AddLine("TOUCH DOWN!", 0.1f, false);
			mPopupInfo->AddLine("AWAY team has scored 6 points.", 0.03f, true);
			mPopupInfo->AddLine("30 seconds removed.", 0.03f, true);
			mPopupInfo->AddPage();
			mAwayScore += 6;
		}
		else if (mBallPlacement > 60)
		{
			mPopupInfo->AddLine("FIELD GOAL!", 0.1f, false);
			mPopupInfo->AddLine("AWAY team has scored 3 points.", 0.03f, true);
			mPopupInfo->AddLine("30 seconds removed.", 0.03f, true);
			mPopupInfo->AddPage();
			mAwayScore += 3;
		}

		ResetAfterPunt();
	}

	InfoLine ln;		char buff[128];
	mPopupInfo->AddLine(strDown[reason], 0.04f, false);
	sprintf(buff, "%s%d yards.", (true == gainedYards) ? "Gain of " : "Loss of ", numYardsGained);		
	mPopupInfo->AddLine(buff, 0.03f, true);

	mPopupInfo->AddPage();


	DTS_LOG_TEXT("%s: %s of %d yards.  Ball on the %d\n", strDown[reason].c_str(), gainedYards ? "Gain" : "Loss", numYardsGained, mLineOfScrimmage);
//	mPopupInfo->Open();

	TheAudioSystem->TestSound_Play(Sounds::Whistle);
	mIsInPlay = false;
	mReadyToPlay = false;
	mIsScoring = true;
}

void CFootballField::ClearFieldAfterPlay(void)
{
	while (false == mPlayers.empty())
	{
		CFootballPlayer::DestroyPlayer(&mPlayers.back());
		mPlayers.pop_back();
	}

	mIsInPlay = false;
}

CFootballPlayer* CFootballField::PlacePlayer(PlayerType type, FootballTeam team, const Vector3 &fieldPos)
{
	CFootballPlayer *newPlayer = nullptr;
	CFootballPlayer::CreatePlayer(&newPlayer, type, team);
	
	if (nullptr != newPlayer)
	{
		newPlayer->mFootballField = this;
		newPlayer->SetPosition(fieldPos);
		mPlayers.push_back(newPlayer);
	}

	return newPlayer;
}

void CFootballField::HikeFootball(void)
{
	//TheAudioSystem->TestSound_Play(Sounds::HikeBall, false);
	Sounds::PlayVO("hike", 3);
	mIsInPlay = true;
}

char CFootballField::FieldPosition(const Vector3 &pos) const { return FieldPosition(pos.z); }
char CFootballField::FieldPosition(float posZ) const
{	//TODO: This is giving the player a bit of an advantage.
	char yard = char((posZ + 0.5f) - (mRowOfYard1 * 1.0f));
	if (yard < 0) yard = 0;
	if (yard > 100) yard = 100;

	return yard;
}

const CFootballPlayer* CFootballField::FindPlayerWithRole(const std::string &roleName)
{
	for (PlayerCItr itr = mPlayers.begin(), itrEnd = mPlayers.end(); itr != itrEnd; ++itr)
		if (roleName == (*itr)->GetRoleName())
			return *itr;
	return nullptr;
}

const CFootballPlayer* CFootballField::GetPlayerWithFootball(void) const
{
	//for (PlayerCItr itr = mPlayers.begin(), itrEnd = mPlayers.end(); itr != itrEnd; ++itr)
	//	if (true == (*itr)->HasFootball())
	//		return *itr;
	//return nullptr;

	return mFootball->GetPlayer();
}

void CFootballField::SpawnDust(const Vector3 &at)
{
	mDustBall->Spawn(at);
}

const Vector3& CFootballField::GetPlayerPosition() const 
{ 
	return mPlayers[0]->GetPosition(); 
}

const Vector3& CFootballField::GetFootballPosition() const 
{
	return mFootball->GetPosition();
}

Vector3 CFootballField::GetCenterAt(char yrd) const
{
	if (-1 == yrd && -1 != mLineOfScrimmage)		
		return GetCenterAt(mLineOfScrimmage);
	return Vector3((mCols) * 0.5f - 0.5f, 0.0f, (yrd + mRowOfYard1) * 1.0f);
}

float CFootballField::LineOfScrimmageZ(void) const
{	
	return GetCenterAt(mLineOfScrimmage).z;
}

float CFootballField::FirstDownZ(void) const
{
	return GetCenterAt(mNextFirstDown).z;
}

void CFootballField::DrawMarkerLine(char yrd, unsigned char tileID) const
{
	for (int x = mColOfLeftEdge + 1; x < mColOfRightEdge; ++x)
		DrawTile(tileID, Vector3(x * 1.0f, 0.0f, (yrd + mRowOfYard1) * 1.0f));
}

void CFootballField::DrawTile(unsigned char tileID, const Vector3 &at) const
{
	float tx = (tileID % 8) * 0.125f + 0.0001f;
	float ty = (tileID / 8) * 0.125f + 0.0001f;
	DrawTile(tx, ty, at);
}

void CFootballField::DrawTile(float tx, float ty, const Vector3 &at) const
{
	CSpriteObject tile(Textures::FieldTiles);
	tile.SetPosition(at);
	tile.GetSprite().SetTextureCoords(tx, ty, 0.125f - 0.0001f, 0.125f - 0.0001f);
	tile.Draw();
}

void CFootballField::ComputeFirstDown(void)
{
	mNextFirstDown = mLineOfScrimmage - 10;
	if (mNextFirstDown < 0)
		mNextFirstDown = 0;
}

void CFootballField::OnTouchDown(void)
{	
	mPopupInfo->AddLine("TOUCH DOWN!", 0.1f, false);
	mPopupInfo->AddLine("Kitten has scored 6 points.", 0.04f, true);
	mPopupInfo->AddPage();
	mHomeScore += 6;
}

void CFootballField::OnSafety(void)
{
	mPopupInfo->AddLine("SAFETY!", 0.1f, false);
	mPopupInfo->AddLine("AWAY team has scored 2 points.", 0.04f, true);
	mPopupInfo->AddPage();
	mHomeScore += 2;
}

void CFootballField::OnGameEnd(void)
{
	if (true == mHasGameConcluded)
		return;

	if (mHomeScore > mAwayScore)
	{
		mHasGameConcluded = true;
		mPopupInfo->AddLine("You Win!", 0.1f, false);
	}
	else if (mAwayScore > mHomeScore)
	{
		mHasGameConcluded = true;
		mPopupInfo->AddLine("You Lose!", 0.1f, false);
	}
	else
	{
		mPopupInfo->AddLine("You Tie!", 0.1f, false);
		ResetAfterPunt();
		mGameTimeLeft = 60.0f;
	}

	int yardsPerPlay = (0 != mTotalDownsPlayed) ? mTotalYardage / mTotalDownsPlayed : 0;

	char buff[128];
	sprintf(buff, "Total Yards: %d / %d per play", mTotalYardage, yardsPerPlay);
	mPopupInfo->AddLine(buff, 0.02f, true);

	sprintf(buff, "Total Plays: %d", mTotalDownsPlayed);
	mPopupInfo->AddLine(buff, 0.02f, true);

	sprintf(buff, "Incomplete / Interception: %d / %d", mIncompletePasses, mTotalInterceptions);
	mPopupInfo->AddLine(buff, 0.02f, true);

	mPopupInfo->AddPage();
	mIsScoring = true;
}