/*~******************************************************************************************
*	~Project:	"Alone"													*
*	~Author:	Tim Beaudet																	*
*	~Modified:	12/16/2011																	*
*	~Purpose:	Base object that will be managed by the world.								*
******************************************************************************************~*/

#ifndef _Alone_CScript_h_
#define _Alone_CScript_h_

#include <string>
#include <sstream>

#define SCRIPT_AUTOWHITE { if (AutoWhite) { mData << AutoWhite; } }

namespace Utilities
{

	class CScript
	{
	public:
		int Version;
		char AutoWhite;

	private:
		std::stringstream mData;
		
	public:

		/*~**********************************************************************************
		*	~Function:	"EndOfFile"															*
		*	~Modified:	4/20/2011															*
		*	~Purpose:	Check if the script has reached the end of the file marker			*
		**********************************************************************************~*/
		bool EndOfFile(void) const { return mData.eof(); }

		/*~**********************************************************************************
		*	~Function:	"Read" "ReadAs" "ReadAsLabeled" "ReadString"						*
		*	~Modified:	4/20/2011															*
		*	~Purpose:	Read data to the next bit of whitespace.							*
		*																					*
		*	~Note:		ReadAsLabeled will read and discard a string, before it				*
		*				reads the actual value.  Label can NOT have whitespace.				*
		*																					*
		*	~Note:		ReadString reads until the '\0' is found.  Allowing white			*
		*				space to be read into the string.									*
		**********************************************************************************~*/
		std::string Read(void);
		template<typename t> t ReadAs(void) { t data; mData >> data; return data; }
		template<typename t> t ReadAsLabeled(void) { std::string lab; mData >> lab; return ReadAs<t>(); }
		std::string ReadString(void);

		/*~**********************************************************************************
		*	~Function:	"Write" "WriteAs" "WriteAsLabeled" "WriteString"					*
		*	~Modified:	4/20/2011															*
		*	~Purpose:	Write data.  Does not append white space unless mAutoWhite			*
		*				is set to a white space character: ' ' '\n' or '\t'.				*
		*				mAutoWhite is added at the end of every Write call.					*
		*																					*
		*	~Note:		WriteAsLabeled will write a string, before it writes the			*
		*				actual data value.  The label can NOT have whitespace.				*
		*																					*
		*	~Note:		WriteString allows the writing of a string with whitespace			*
		*				but ReadString must be called to read in the whitespace				*
		*				during a load.  This will input the '\0' into the script.			*
		**********************************************************************************~*/
		void Write(const std::string &data);
		template<typename t> void WriteAs(t data) { mData << data; SCRIPT_AUTOWHITE; }
		template<typename t> void WriteAsLabeled(const std::string &strLabel, t data) { mData << strLabel;	mData << ' '; WriteAs(data); }
		void WriteString(const std::string &str);

		/*~**********************************************************************************
		*	~Function:	"SaveToFile" "SaveToData"											*
		*	~Modified:	4/20/2011															*
		*	~Purpose:	Save the script to file or data, pending your needs.				*
		**********************************************************************************~*/
		bool SaveToFile(const std::string &strFilePath) const;
		bool SaveToData(std::string &strData) const;

		/*~**********************************************************************************
		*	~Function:	"OpenScriptFromFile" "OpenScriptFromData"							*
		*	~Modified:	4/20/2011															*
		*	~Purpose:	Opens a script from file, or places the block of data into the		*
		*				script for reading back.											*
		*																					*
		*	~Note:		You must call CloseScript() to clean up when finished.				*
		**********************************************************************************~*/
		friend CScript* OpenScriptFromFile(const std::string &strFilePath);
		friend CScript* OpenScriptFromData(const std::string &strData);

		/*~**********************************************************************************
		*	~Function:	"CloseScript"														*
		*	~Modified:	4/20/2011															*
		*	~Purpose:	Close the script and destroy any/all resource data.  The			*
		*				script is unusable after this call.  This does not close			*
		*				a file associated with the script, but deletes all data.			*
		**********************************************************************************~*/
		friend void CloseScript(CScript *script);

	private:
		CScript(void);
		CScript(const CScript &obj);
		~CScript(void);
		CScript& operator=(const CScript &obj);
	};

};	//namespace Utilities

#endif _Alone_CScript_h_